@@ -20,6 +20,8 @@ import DataFrame.Operations.Typing as D
 import DataFrame.Operations.Aggregation as D
 import DataFrame.Display.Terminal.Plot as D
 import DataFrame.IO.CSV as D
+-- Support for Parquet is still experimental
+import DataFrame.IO.Parquet as D
 
 import Data.Function
 
